@@ -35,4 +35,16 @@ enum {
     STREAM_VIDEO_FILTER
 };
 
+static inline int bitmap_fmt_is_rgb(uint8_t fmt)
+{
+    static const int BITMAP_FMT_IS_RGB[SPICE_BITMAP_FMT_ENUM_END] =
+	                                {0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1};
+
+    if (fmt >= SPICE_BITMAP_FMT_ENUM_END) {
+        spice_warning("fmt >= SPICE_BITMAP_FMT_ENUM_END");
+        return 0;
+    }
+    return BITMAP_FMT_IS_RGB[fmt];
+}
+
 #endif
