@@ -10,7 +10,6 @@
 #include <stdlib.h>
 #include "test_display_base.h"
 
-SpiceServer *server;
 SpiceCoreInterface *core;
 SpiceTimer *ping_timer;
 
@@ -38,11 +37,13 @@ int simple_commands[] = {
 
 int main(void)
 {
+    Test *test;
+
     core = basic_event_loop_init();
-    server = test_init(core);
+    test = test_new(core);
     //spice_server_set_image_compression(server, SPICE_IMAGE_COMPRESS_OFF);
-    test_add_display_interface(server);
-    test_set_simple_command_list(simple_commands, COUNT(simple_commands));
+    test_add_display_interface(test);
+    test_set_simple_command_list(test, simple_commands, COUNT(simple_commands));
 
     ping_timer = core->timer_add(pinger, NULL);
     core->timer_start(ping_timer, ping_ms);
